@@ -1,4 +1,4 @@
-# $Id: Cart.pm 1318 2006-07-10 23:42:32Z claco $
+# $Id: Cart.pm 1353 2006-08-04 13:04:17Z claco $
 package Catalyst::Helper::Model::Handel::Cart;
 use strict;
 use warnings;
@@ -37,7 +37,7 @@ use Test::More tests => 2;
 use strict;
 use warnings;
 
-use_ok(Catalyst::Test, '[% app %]');
+use_ok('Catalyst::Test', '[% app %]');
 use_ok('[% class %]');
 __END__
 
